using System.Collections.Generic;
using System.Threading.Tasks;
using GxPress.Repository.Interface.Navigation;
using GxPress.Result.Navigation;
using GxPress.Service.Interface.Navigation;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;

namespace GxPress.Api.WebControllers
{
    /// <summary>
    /// 导航数据
    /// </summary>
    [Route("api/web/navigation")]
    [ApiController]
    [Authorize]
    public class NavigationController : Controller
    {
        private readonly INavigationRepository navigationRepository;
        private readonly INavigationService navigationService;
        public NavigationController(INavigationRepository navigationRepository, INavigationService navigationService)
        {
            this.navigationRepository = navigationRepository;
            this.navigationService = navigationService;
        }
        /// <summary>
        /// 获取导航栏数据
        /// </summary>
        /// <returns></returns>
        [HttpGet("media/{id}")]
        [AllowAnonymous]
        public async Task<MiddleLableResult> GetNavigationResults(int id)
        {
            return await navigationService.GetNavigationResults(id);
        }
        /// <summary>
        /// 获取中间页面数据
        /// </summary>
        /// <returns></returns>
        [HttpGet("middle-lable/{middleLableId}")]
        [AllowAnonymous]
        public async Task<MiddleLableResult> GetMiddleLableResults(int middleLableId)
        {
            return await navigationService.GetMiddleLableResults(middleLableId);
        }

        /// <summary>
        /// 列表 pc 1 移动 2
        /// </summary>
        /// <returns></returns>
        [HttpGet("{terminal}")]
        [AllowAnonymous]
        public async Task<IEnumerable<NavigationResult>> GetList(int terminal)
        {
            return await navigationRepository.GetAllAsync(terminal);
        }
        /// <summary>
        /// 标签查询数据
        /// </summary>
        /// <returns></returns>
        [HttpGet("label/{labelId}")]
        [AllowAnonymous]
        public async Task<LabelMediaResult> GetLabelMediaResult(int labelId)
        {
            return await navigationService.GetLabelMediaResult(labelId);
        }
    }
}