﻿using System;
using Datory.Annotations;
using GxPress.EnumConst;

namespace GxPress.Entity.tede2.Media
{
    /// <summary>
    /// 媒体类型 文章、书籍、视频、音频、期刊
    /// </summary>
    [DataTable("tede_media")]
    public class Media : Datory.Entity
    {
        /// <summary>
        /// 编号
        /// </summary>
        /// <value></value>
        [DataColumn]
        public string AutoNumber { get; set; }
        /// <summary>
        /// 媒体类型
        /// Article=1,
        //书籍
        //Book = 2,
        ////课程
        //Curriculum = 3,
        ////音频
        //Audio = 4,
        ////期刊
        //Journal = 5,
        ////视频
        //Video=6
        /// </summary>
        [DataColumn]
        public int MediaType { get; set; }
        /// <summary>
        /// 标题
        /// </summary>
        [DataColumn]
        public string Title { get; set; }

        /// <summary>
        /// 标题图片
        /// </summary>
        [DataColumn]
        public string ImageUrls { get; set; }

        /// <summary>
        /// 简介
        /// </summary>
        [DataColumn]
        public string Summary { get; set; }

        /// <summary>
        /// 作者
        /// </summary>
        [DataColumn]
        public string Author { get; set; }
        /// <summary>
        /// 名师ID
        /// </summary>
        /// <value></value>
        [DataColumn]
        public int TeacherId { get; set; }
        /// <summary>
        /// 来源
        /// </summary>
        [DataColumn]
        public string Source { get; set; }

        /// <summary>
        /// Json内容
        /// </summary>
        [DataColumn(Text = true)]
        public string Blocks { get; set; }

        /// <summary>
        /// 审核状态
        /// </summary>
        [DataColumn]
        public bool IsChecked { get; set; }

        /// <summary>
        /// 添加时间
        /// </summary>
        [DataColumn]
        public DateTime AddDate { get; set; }
        /// <summary>
        /// 排序
        /// </summary>
        [DataColumn]
        public int Sort { get; set; }

        /// <summary>
        /// 是否置顶
        /// </summary>
        [DataColumn]
        public bool IsTop { get; set; }
        /// <summary>
        /// 分类ID
        /// </summary>
        [DataColumn]
        public string CategoryId { get; set; }
        /// <summary>
        /// 分类ID
        /// </summary>
        [DataColumn]
        public int AppCategoryId { get; set; }
        /// <summary>
        /// 完整分类ID
        /// </summary>
        [DataColumn]
        public string FullCategoryId { get; set; }
        /// <summary>
        /// 分类名称
        /// </summary>
        [DataColumn]
        public string CategoryName { get; set; }
        /// <summary>
        /// 标签ID
        /// </summary>
        [DataColumn]
        public int LableId { get; set; }
        /// <summary>
        /// 是否推荐
        /// </summary>
        [DataColumn]
        public bool IsRecommend { get; set; }
        /// <summary>
        /// 出版社
        /// </summary>
        [DataColumn]
        public string Press { get; set; }
        /// <summary>
        /// 出版时间
        /// </summary>
        [DataColumn]
        public string PublishTime { get; set; }

        /// <summary>
        /// 价格
        /// </summary>
        [DataColumn]
        public decimal Price { get; set; }
        /// <summary>
        /// IOS价格
        /// </summary>
        [DataColumn]
        public decimal IosPrice { get; set; }
        /// <summary>
        /// 其他价格
        /// </summary>
        [DataColumn]
        public decimal OtherPrice { get; set; }
        /// <summary>
        /// 操作人ID
        /// </summary>
        /// <value></value>
        [DataColumn]
        public int AdminId { get; set; }

        /// <summary>
        /// 操作人
        /// </summary>
        /// <value></value>
        [DataColumn]
        public string Creator { get; set; }

        /// <summary>
        /// ios折扣价格打折
        /// </summary>
        [DataColumn]
        public decimal IosDiscountPrice { get; set; }
        /// <summary>
        /// 其他折扣价格打折
        /// </summary>
        [DataColumn]
        public decimal OtherDiscountPrice { get; set; }

        /// <summary>
        /// 免费类容占比
        /// </summary>
        /// <value></value>
        [DataColumn]
        public decimal FreeProportion { get; set; }
        /// <summary>
        /// 是否删除
        /// </summary>
        /// <value></value>
        [DataColumn]
        public bool IsDelete { get; set; }

        /// <summary>
        /// 类型 名栏 公众号 期刊
        /// </summary>
        /// <value></value>
        [DataColumn]
        public AttachTypeConst AttachType { get; set; }

        /// <summary>
        /// 类型 1名栏 2公众号 
        /// </summary>
        /// <value></value>
        [DataColumn]
        public int AttachId { get; set; }
        /// <summary>
        /// 关联的媒体ID
        /// </summary>
        /// <value></value>
        [DataColumn]
        public int MediaId { get; set; }
        /// <summary>
        /// 关联的媒体标题
        /// </summary>
        /// <value></value>
        [DataColumn]
        public string MediaTitle { get; set; }
        /// <summary>
        /// 是否有库
        /// </summary>
        /// <value></value>
        [DataColumn]
        public bool IsLibrary { get; set; }
        /// <summary>
        /// 刊期年
        /// </summary>
        /// <value></value>
        [DataColumn]
        public string JournalsYear { get; set; }
        /// <summary>
        /// 刊期期数
        /// </summary>
        /// <value></value>
        [DataColumn]
        public int JournalsCategory { get; set; }
        /// <summary>
        /// 收藏次数
        /// </summary>
        [DataColumn]
        public int CollectCount { get; set; }
        /// <summary>
        /// 转发数量
        /// </summary>
        /// <value></value>
        [DataColumn]
        public int RetransmissionCount { get; set; }
        /// <summary>
        /// 浏览量
        /// </summary>
        [DataColumn]
        public int ReadCount { get; set; }
        /// <summary>
        /// 评论数量
        /// </summary>
        /// <value></value>
        [DataColumn]
        public int CommentCount { get; set; }
        /// <summary>
        /// 销售总金额
        /// </summary>
        /// <value></value>
        [DataColumn]
        public decimal SellAmount { get; set; }
        /// <summary>
        /// 订单数量
        /// </summary>
        /// <value></value>
        [DataColumn]
        public int OrderCount { get; set; }
        /// <summary>
        /// 评论分数
        /// </summary>
        /// <value></value>
        [DataColumn]
        public decimal CommentScore { get; set; }
        /// <summary>
        /// 是否习题
        /// </summary>
        /// <value></value>
        [DataColumn]
        public bool IsExercises { get; set; }
    }
}
