﻿using System;
using System.Collections.Generic;
using GxPress.Common.Page;

namespace GxPress.Request.Media
{
    /// <summary>
    /// 媒体
    /// </summary>
    public class MediaRequest
    {
        /// <summary>
        /// 媒体类型
        /// Article=1,
        //书籍
        //Book = 2,
        ////课程
        //Curriculum = 3,
        ////音频
        //Audio = 4,
        ////期刊
        //Journal = 5,
        ////视频
        //Video=6
        /// </summary>

        public int MediaType { get; set; }
        /// <summary>
        /// 标题
        /// </summary>

        public string Title { get; set; }

        /// <summary>
        /// 标题图片
        /// </summary>

        public string ImageUrls { get; set; }

        /// <summary>
        /// 简介
        /// </summary>

        public string Summary { get; set; }

        /// <summary>
        /// 作者
        /// </summary>

        public string Author { get; set; }

        /// <summary>
        /// 来源
        /// </summary>

        public string Source { get; set; }

        /// <summary>
        /// Json内容
        /// </summary>
        public string Blocks { get; set; }

        /// <summary>
        /// 审核状态
        /// </summary>

        public bool IsChecked { get; set; }

        /// <summary>
        /// 添加时间
        /// </summary>

        public DateTime AddDate { get; set; }
        /// <summary>
        /// 排序
        /// </summary>

        public int Sort { get; set; }
        /// <summary>
        /// 文章阅读量
        /// </summary>

        public int ReadCount { get; set; }
        /// <summary>
        /// 是否置顶
        /// </summary>

        public bool IsTop { get; set; }
        /// <summary>
        /// 分类ID
        /// </summary>

        public int CategoryId { get; set; }
        /// <summary>
        /// 标签ID
        /// </summary>

        public int LableId { get; set; }
        /// <summary>
        /// 是否推荐
        /// </summary>

        public bool IsRecommend { get; set; }
        /// <summary>
        /// 出版社
        /// </summary>

        public string Press { get; set; }
        /// <summary>
        /// 出版时间
        /// </summary>

        public string PublishTime { get; set; }

        /// <summary>
        /// 媒体类
        /// </summary>
        public List<MediaLibraryRequest> MediaLibraryRequest { get; set; }
        /// <summary>
        /// 媒体标签
        /// </summary>
        public List<MediaLableRequest> MediaLableRequests { get; set; }
    }
    /// <summary>
    /// 媒体库
    /// </summary>
    public class MediaLibraryRequest
    {
        /// <summary>
        /// 媒体ID
        /// </summary>
        /// <value></value>
        public int MediaId { get; set; }
        /// <summary>
        /// 文件路径
        /// </summary>
        public string FileUrl { get; set; }
        /// <summary>
        /// 文件名称
        /// </summary>

        public string Name { get; set; }
        /// <summary>
        /// 排序
        /// </summary>

        public int Sort { get; set; }
    }

    /// <summary>
    /// 媒体标签
    /// </summary>
    public class MediaLableRequest
    {
        /// <summary>
        /// 媒体ID
        /// </summary>
        /// <value></value>
        public int MediaId { get; set; }
        /// <summary>
        /// 标签ID
        /// </summary>
        public string LableId { get; set; }
        /// <summary>
        /// 标签名称
        /// </summary>
        public string LableName { get; set; }
    }
    /// <summary>
    /// 
    /// </summary>
    public class MediaSearchRequest : PageParameter
    {
        /// <summary>
        /// 0全部
        /// </summary>
        /// <value></value>
        public int MediaType { get; set; }
        /// <summary>
        /// 0 全部 1删除 2未删除
        /// </summary>
        /// <value></value>
        public int IsDelete { get; set; }

        /// <summary>
        /// 0 全部 1 包含 2 未包含
        /// </summary>
        /// <value></value>
        public int LibraryType { get; set; }
        /// <summary>
        /// 搜索字段
        /// </summary>
        /// <value></value>
        public string KeyWord { get; set; }
        /// <summary>
        /// 开始时间
        /// </summary>
        /// <value></value>
        public string BeginTime { get; set; }
        /// <summary>
        /// 结束时间
        /// </summary>
        /// <value></value>
        public string EndTime { get; set; }
        /// <summary>
        /// 是否上下架 0全部 1 上架 2下架
        /// </summary>
        /// <value></value>
        public int IsChecked { get; set; }

    }
    /// <summary>
    /// 
    /// </summary>
    public class BookCatalogRequest
    {
        /// <summary>
        /// 媒体ID
        /// </summary>
        /// <value></value>
        public int MediaId { get; set; }
        /// <summary>
        /// 目录名称
        /// </summary>
        /// <value></value>
        public string CatalogNameg { get; set; }
        /// <summary>
        /// 目录ID
        /// </summary>
        /// <value></value>
        public string CatalogId { get; set; }
        /// <summary>
        /// 路径
        /// </summary>
        /// <value></value>
        public string Path { get; set; }
        /// <summary>
        /// 用户ID
        /// </summary>
        /// <value></value>
        public int UserId { get; set; }
    }
    /// <summary>
    /// 排行榜
    /// </summary>
    public class RankingListRequest : Common.Page.PageParameter
    {
        /// <summary>
        ///  1 热搜 2热书榜 3 课程榜 4Top 100 新书 5文献头条  6试听榜 7大众热评 8Top 100 专著
        /// </summary>
        /// <value></value>
        public int DataTypeValue { get; set; }
        /// <summary>
        /// 搜查
        /// </summary>
        /// <value></value>
        public string KeyWord { get; set; }
    }
    /// <summary>
    /// 
    /// </summary>
    public class MediaIdsRequest
    {
        public int LableId { get; set; }
        public List<int> Ids { get; set; }

        public int Limit { get; set; }
    }
    /// <summary>
    /// 
    /// </summary>
    public class MediaLableIdRequest : Common.Page.PageParameter
    {
        public int LableId { get; set; }
        public List<int> Ids { get; set; }
    }

    /// <summary>
    /// 每日微课
    /// </summary>
    public class MediaDayWeBookRequest : Common.Page.PageParameter
    {
        /// <summary>
        /// 名称
        /// </summary>
        /// <value></value>
        public string Name { get; set; }
        /// <summary>
        /// 类型
        /// </summary>
        /// <value></value>
        public int TypeValue { get; set; }
    }
    /// <summary>
    /// 内容导航
    /// </summary>
    public class MediaContentNavigationRequest : Common.Page.PageParameter
    {
        /// <summary>
        /// 导航ID
        /// </summary>
        /// <value></value>
        public int CategoryId { get; set; }
        /// <summary>
        /// 媒体ID
        /// </summary>
        /// <value></value>
        public List<int> MediaIds { get; set; }
        /// <summary>
        /// 作者ID
        /// </summary>
        /// <value></value>
        public List<int> TeacherId { get; set; }
        /// <summary>
        /// 开始时间
        /// </summary>
        /// <value></value>
        public string BeginTime { get; set; }
        /// <summary>
        /// 结束时间
        /// </summary>
        /// <value></value>
        public string EndTime { get; set; }
        /// <summary>
        /// 出版社
        /// </summary>
        /// <value></value>
        public List<string> Press { get; set; }
    }

    /// <summary>
    /// 明栏
    /// </summary>
    public class MediaStarLableRequest : Common.Page.PageParameter
    {
        /// <summary>
        /// 
        /// </summary>
        /// <value></value>
        public int Id { get; set; }
        /// <summary>
        /// 媒体ID
        /// </summary>
        /// <value></value>
        public List<int> MediaIds { get; set; }
        /// <summary>
        /// 关键字
        /// </summary>
        /// <value></value>
        public string KeyWord { get; set; }
    }
}

