using System.Collections.Generic;
using System.Threading.Tasks;
using AutoMapper;
using Datory;
using GxPress.Common.AppOptions;
using GxPress.Common.Tools;
using GxPress.EnumConst;
using GxPress.Repository.Interface.AppChannel;
using GxPress.Result.AppChannel;
using Microsoft.Extensions.Options;

namespace GxPress.Repository.Implement.AppChannel
{
    public class AppChannelRepository : IAppChannelRepository
    {
        private readonly Repository<Entity.tede2.AppChannel.AppChannel> _repository;
        private readonly IMapper _mapper;

        public AppChannelRepository(IOptionsMonitor<DatabaseOptions> dbOptionsAccessor, IMapper mapper)
        {
            var databaseType = StringUtils.ToEnum<DatabaseType>(dbOptionsAccessor.CurrentValue.DatabaseType, DatabaseType.MySql);
            var database = new Database(databaseType, dbOptionsAccessor.CurrentValue.ConnectionString);
            _repository = new Repository<Entity.tede2.AppChannel.AppChannel>(database);
            _mapper = mapper;
        }

        public IDatabase Database => _repository.Database;
        public string TableName => _repository.TableName;
        public List<TableColumn> TableColumns => _repository.TableColumns;
        /// <summary>
        /// 获取详情
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<Entity.tede2.AppChannel.AppChannel> GetAsync(int id)
        {
            var result = await _repository.GetAsync(id);
            if (result != null)
                result.ImageUrls = StringUtils.AddDomain(result.ImageUrls);
            return result;
        }

        /// <summary>
        /// 查询
        /// </summary>
        /// <returns></returns>
        public async Task<IEnumerable<AppChannelResult>> GetAllAsync(AppChannelConst channelTypeValue)
        {
            var result = await _repository.GetAllAsync<AppChannelResult>(Q.Where(nameof(Entity.tede2.AppChannel.AppChannel.ChannelTypeValue), channelTypeValue).OrderBy(nameof(Entity.tede2.AppChannel.AppChannel.Sort)));
            foreach (var item in result)
                item.ImageUrls = StringUtils.AddDomain(item.ImageUrls);
            return result;
        }
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<bool> DeleteAsync(int id)
        {
            return await _repository.DeleteAsync(id);
        }
        /// <summary>
        /// 修改
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public async Task<bool> UpdateAsync(Entity.tede2.AppChannel.AppChannel request)
        {
            var model = await _repository.GetAsync(request.Id);
            if (!string.IsNullOrEmpty(request.ImageUrls))
                model.ImageUrls = StringUtils.RemoveDomain(request.ImageUrls);
            model.IsDisable = request.IsDisable;
            model.TypeValue = request.TypeValue;
            model.Name = request.Name;
            model.Sort = request.Sort;
            model.IsDisable = request.IsDisable;
            return await _repository.UpdateAsync(model);
        }
        /// <summary>
        /// 添加
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public async Task<bool> InsertAsync(Entity.tede2.AppChannel.AppChannel model)
        {
            if (!string.IsNullOrEmpty(model.ImageUrls))
                model.ImageUrls = StringUtils.RemoveDomain(model.ImageUrls);
            return await _repository.InsertAsync(model) > 0;
        }
    }
}