using System.Collections.Generic;
using System.Threading.Tasks;
using AutoMapper;
using Datory;
using GxPress.Common.AppOptions;
using GxPress.Common.Tools;
using GxPress.Repository.Interface.AppChannel;
using GxPress.Request.AppChannel;
using Microsoft.Extensions.Options;
using System.Transactions;
namespace GxPress.Repository.Implement.AppChannel
{
    public class ChannelCategoryMediaRepository : IChannelCategoryMediaRepository
    {
        private readonly Repository<Entity.tede2.AppChannel.ChannelCategoryMedia> _repository;
        private readonly IMapper _mapper;
        public ChannelCategoryMediaRepository(IOptionsMonitor<DatabaseOptions> dbOptionsAccessor, IMapper mapper)
        {
            var databaseType = StringUtils.ToEnum<DatabaseType>(dbOptionsAccessor.CurrentValue.DatabaseType, DatabaseType.MySql);
            var database = new Database(databaseType, dbOptionsAccessor.CurrentValue.ConnectionString);
            _repository = new Repository<Entity.tede2.AppChannel.ChannelCategoryMedia>(database);
            _mapper = mapper;
        }
        public IDatabase Database => _repository.Database;
        public string TableName => _repository.TableName;
        public List<TableColumn> TableColumns => _repository.TableColumns;

        /// <summary>
        /// 获取详情
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<Entity.tede2.AppChannel.ChannelCategoryMedia> GetAsync(int id)
        {
            var result = await _repository.GetAsync(id);
            return result;
        }
        /// <summary>
        /// 查询 获取MediaId
        /// </summary>
        /// <returns></returns>
        /// <returns>isCategory</returns>
        public async Task<IEnumerable<int>> GetAllAsync(int categoryId,bool isCategory=true)
        {
            var query = Q.NewQuery();
            query.Select(nameof(Entity.tede2.AppChannel.ChannelCategoryMedia.MediaId));
            query.Where(nameof(Entity.tede2.AppChannel.ChannelCategoryMedia.ChannelCategoryId), categoryId);
             query.Where(nameof(Entity.tede2.AppChannel.ChannelCategoryMedia.IsCategory), isCategory);
            var result = await _repository.GetAllAsync<int>(query);
            return result;
        }
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<bool> DeleteAsync(int id)
        {
            return await _repository.DeleteAsync(id);
        }
        /// <summary>
        /// 添加
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public async Task<bool> InsertAsync(ChannelCategoryMediaInRequest request)
        {
            try
            {
                using (var tran = new TransactionScope())
                {
                    //删除
                    await _repository.DeleteAsync(Q.Where(nameof(Entity.tede2.AppChannel.ChannelCategoryMedia.ChannelCategoryId),request.CategoryId));
                    foreach (var item in request.MediaIds)
                    {
                        await _repository.InsertAsync(new Entity.tede2.AppChannel.ChannelCategoryMedia { ChannelCategoryId = request.CategoryId, MediaId = item, IsCategory = request.IsCategory });
                    }
                    tran.Complete();
                }
            }
            catch
            {
                return false;
            }
            return true;
        }
    }
}