using GxPress.Repository.Interface.AppChannel;
using System.Collections.Generic;
using AutoMapper;
using Datory;
using GxPress.Common.AppOptions;
using GxPress.Common.Tools;
using Microsoft.Extensions.Options;
using System.Threading.Tasks;
using GxPress.Entity.tede2.AppChannel;
using GxPress.Result.AppChannel;
using System.Linq;
using GxPress.Common.Page;
using GxPress.Request.AppChannel;

namespace GxPress.Repository.Implement.AppChannel
{
    public class ChannelCategoryRepository : IChannelCategoryRepository
    {
        private readonly Repository<Entity.tede2.AppChannel.ChannelCategory> _repository;
        private readonly IMapper _mapper;

        public ChannelCategoryRepository(IOptionsMonitor<DatabaseOptions> dbOptionsAccessor, IMapper mapper)
        {
            var databaseType = StringUtils.ToEnum<DatabaseType>(dbOptionsAccessor.CurrentValue.DatabaseType, DatabaseType.MySql);
            var database = new Database(databaseType, dbOptionsAccessor.CurrentValue.ConnectionString);
            _repository = new Repository<Entity.tede2.AppChannel.ChannelCategory>(database);
            _mapper = mapper;
        }
        public IDatabase Database => _repository.Database;
        public string TableName => _repository.TableName;
        public List<TableColumn> TableColumns => _repository.TableColumns;
        /// <summary>
        /// 获取详情
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<Entity.tede2.AppChannel.ChannelCategory> GetAsync(int id)
        {
            var result = await _repository.GetAsync(id);
            return result;
        }
        /// <summary>
        /// 查询 获取频道分类
        /// </summary>
        /// <returns></returns>
        public async Task<IEnumerable<ChannelCategoryResult>> GetAllAsync(int appChannelId)
        {
            var result = new List<ChannelCategoryResult>();
            var query = Q.NewQuery();
            query.Where(nameof(Entity.tede2.AppChannel.ChannelCategory.AppChannelId), appChannelId);
            var items = await _repository.GetAllAsync<ChannelCategoryResult>(query);
            result.AddRange(items.Where(n => n.ParentId == 0));
            foreach (var item in result)
            {
                item.IsChildren = items.Any(n => n.ParentId == item.Id);
                if (item.IsChildren)
                    item.Children = items.Where(n => n.ParentId == item.Id);
            }
            return result;
        }
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<bool> DeleteAsync(int id)
        {
            return await _repository.DeleteAsync(id);
        }
        /// <summary>
        /// 添加
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public async Task<bool> InsertAsync(ChannelCategory request)
        {
            return await _repository.InsertAsync(request) > 0;
        }
        /// <summary>
        ///修改导航分类
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public async Task<bool> UpdateAsync(ChannelCategory request)
        {
            return await _repository.UpdateAsync(request);
        }
        public async Task<bool> IsChildrenAsync(int id)
        {
            return await _repository.ExistsAsync(Q.Where(nameof(Entity.tede2.AppChannel.ChannelCategory.ParentId), id));
        }

        /// <summary>
        /// 根据父级获取 获取频道分类
        /// </summary>
        /// <returns></returns>
        public async Task<PagedList<ChannelCategoryResult>> GetAllByParentIdAsync(AppChannelSearchRequest request)
        {
            var result = new PagedList<ChannelCategoryResult>();
            var query = Q.NewQuery();
            query.Where(nameof(Entity.tede2.AppChannel.ChannelCategory.ParentId), request.CategoryId);
            result.Total = await _repository.CountAsync(query);
            query.ForPage(request.Page, request.PerPage);
            result.Items = await _repository.GetAllAsync<ChannelCategoryResult>(query);
            foreach (var item in result.Items)
            {
                item.IsCategory = true;
            }
            return result;
        }
    }
}