using System.Collections.Generic;
using System.Threading.Tasks;
using AutoMapper;
using Datory;
using GxPress.Common.AppOptions;
using GxPress.Common.Tools;
using GxPress.Repository.Interface.Group;
using Microsoft.Extensions.Caching.Distributed;
using Microsoft.Extensions.Options;
using System.Transactions;
using System;
using System.Linq;
using Dapper;

namespace GxPress.Repository.Implement.Group
{
    public class UserGroupCategoryRepository : IUserGroupCategoryRepository
    {
        private readonly Repository<Entity.tede2.Group.UserGroupCategory> _repository;
        private readonly IMapper _mapper;
        private readonly IDistributedCache _cache;
        private readonly string _connectionString;
        private readonly string _databaseTypestr;
        public UserGroupCategoryRepository(IOptionsMonitor<DatabaseOptions> dbOptionsAccessor, IMapper mapper, IDistributedCache cache)
        {
            _databaseTypestr = dbOptionsAccessor.CurrentValue.DatabaseType;
            _connectionString = dbOptionsAccessor.CurrentValue.ConnectionString;
            var databaseType = StringUtils.ToEnum<DatabaseType>(dbOptionsAccessor.CurrentValue.DatabaseType, DatabaseType.MySql);
            var database = new Database(databaseType, dbOptionsAccessor.CurrentValue.ConnectionString);
            _mapper = mapper;
            _cache = cache;
            _repository = new Repository<Entity.tede2.Group.UserGroupCategory>(database);
        }

        public IDatabase Database => _repository.Database;
        public string TableName => _repository.TableName;
        public List<TableColumn> TableColumns => _repository.TableColumns;
        /// <summary>
        /// 添加
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> InsertAsync(IEnumerable<Entity.tede2.Group.UserGroupCategory> request, int userId)
        {
            if (request == null || request.Count() == 0)
                return true;
            try
            {
                using (var tran = new TransactionScope())
                {
                    await _repository.DeleteAsync(Q.Where(nameof(Entity.tede2.Group.UserGroupCategory.UserId), userId));
                    string sql = $@"INSERT INTO `tede_user_group_category`
                                                                (
                                                                `Guid`,
                                                                `CreatedDate`,
                                                                `GroupCategroyId`,
                                                                `UserId`,
                                                                `LastModifiedDate`)
                                                                 VALUES";
                    foreach (var item in request)
                    {
                        var guid = System.Guid.NewGuid().ToString();
                        var createdDateValue = DateTime.Now.ToString();
                        sql += $@"('{guid}','{createdDateValue}',{item.GroupCategroyId},{userId},'{createdDateValue}'),";
                    }
                    sql = sql.Remove(sql.Length - 1, 1);
                    var databaseType = _databaseTypestr.ToEnum<DatabaseType>(DatabaseType.MySql);
                    var database = new Database(databaseType, _connectionString);
                    var connection = database.GetConnection();
                    var result = await connection.ExecuteAsync(sql);
                    tran.Complete();
                }
            }
            catch
            {
                return false;
            }
            return true;
        }

        public async Task<bool> IsExistsAsync(int userId)
        {
            return await _repository.ExistsAsync(Q.Where(nameof(Entity.tede2.Group.UserGroupCategory.UserId), userId));
        }

    
    }
}