using System.Collections.Generic;
using System.Threading.Tasks;
using AutoMapper;
using Datory;
using GxPress.Common.AppOptions;
using GxPress.Common.Tools;
using GxPress.Repository.Interface.Navigation;
using Microsoft.Extensions.Options;
using System.Transactions;
using GxPress.Request.Navigation;

namespace GxPress.Repository.Implement.Navigation
{
    public class MiddleLableNexusRepository : IMiddleLableNexusRepository
    {
        private readonly Repository<Entity.Navigations.MiddleLableNexus> _repository;
        private readonly IMapper _mapper;
        public MiddleLableNexusRepository(IOptionsMonitor<DatabaseOptions> dbOptionsAccessor, IMapper mapper)
        {
            var databaseType =
                StringUtils.ToEnum<DatabaseType>(dbOptionsAccessor.CurrentValue.DatabaseType, DatabaseType.MySql);
            var database = new Database(databaseType, dbOptionsAccessor.CurrentValue.ConnectionString);
            _repository = new Repository<Entity.Navigations.MiddleLableNexus>(database);
            _mapper = mapper;
        }

        public IDatabase Database => _repository.Database;
        public string TableName => _repository.TableName;
        public List<TableColumn> TableColumns => _repository.TableColumns;
        /// <summary>
        /// 添加
        /// </summary>
        /// <param name="middleLableId"></param>
        /// <param name="ids"></param>
        /// <returns></returns>
        public async Task<bool> InsertAsync(List<MiddleLableNexusUpRequest> middleLableNexusUpRequests)
        {
            try
            {
                using (var transactionScope = new TransactionScope())
                {
                    foreach (var item in middleLableNexusUpRequests)
                    {

                        await _repository.InsertAsync(_mapper.Map<Entity.Navigations.MiddleLableNexus>(item));
                    }
                    transactionScope.Complete();
                }
            }
            catch
            {
                return false;
            }
            return true;
        }

        /// <summary>
        /// 查询
        /// </summary>
        /// <param name="middleLableId"></param>
        /// <returns></returns>
        public async Task<IEnumerable<Entity.Navigations.MiddleLableNexus>> GetAllTaskAsync(int middleLableId)
        {
            var query = Q.NewQuery();
            query.Where(nameof(Entity.Navigations.MiddleLableNexus.MiddleLableId), middleLableId);
            query.OrderByDesc(nameof(Entity.Navigations.MiddleLableNexus.Sort));
            return await _repository.GetAllAsync(query);
        }
        public async Task<bool> DeleteByMiddleLableIdAsync(int middleLableId)
        {
            return await _repository.DeleteAsync(Q.Where(nameof(Entity.Navigations.MiddleLableNexus.MiddleLableId), middleLableId)) > 0;
        }

        
        public async Task<bool> UpdateAsync(int middleLableId, List<MiddleLableNexusUpRequest> middleLableNexusUpRequests)
        {
            try
            {
                await DeleteByMiddleLableIdAsync(middleLableId);

                await InsertAsync(middleLableNexusUpRequests);
            }
            catch
            {
                return false;

            }
            return true;
        }

    }
}