using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using GxPress.Repository.Interface.Category;
using GxPress.Repository.Interface.SystemLabel;
using GxPress.Result.Category;
using GxPress.Service.Interface.Category;
namespace GxPress.Service.Implement.Category
{
    public class CategoryService : ICategoryService
    {
        private readonly ICategoryRepository categoryRepository;
        private readonly ISystemLableMediaRepository systemLableMediaRepository;
        public CategoryService(ICategoryRepository categoryRepository, ISystemLableMediaRepository systemLableMediaRepository)
        {
            this.categoryRepository = categoryRepository;
            this.systemLableMediaRepository = systemLableMediaRepository;
        }
        /// <summary>
        /// 内容导航分类
        /// </summary>
        /// <param name="typeValue"></param>
        /// <returns></returns>
        public async Task<IEnumerable<CategoryResult>> GetCategoryAllAsync(int typeValue = 0)
        {
            var result = await categoryRepository.GetCategoryAllAsync(typeValue);
            foreach (var item in result)
            {
                if (!item.IsChildren)
                {
                    var mediaIds = await systemLableMediaRepository.GetMediaIdsAsync(item.Id, 1);
                    item.MediaIds = mediaIds.ToList();
                }
            }
            return result;
        }
        public async Task<IEnumerable<CategoryResult>> GetCategoryChildrenAsync(int parentId, int typeValue)
        {
            var result = await categoryRepository.GetCategoryChildrenAsync(parentId, typeValue);
            foreach (var item in result)
            {
                if (!item.IsChildren)
                {
                    var mediaIds = await systemLableMediaRepository.GetMediaIdsAsync(item.Id, 1);
                    item.MediaIds = mediaIds.ToList();
                }
            }
            return result;
        }

        public async Task<CategoryResult> GetAsync(int id)
        {
            var result = await categoryRepository.GetAsync(id);
            if (result != null)
            {
                var mediaIds = await systemLableMediaRepository.GetMediaIdsAsync(id,1);
                result.MediaIds = mediaIds.ToList();
            }
            return result;
        }
    }
}