using System.Collections.Generic;
using System.Threading.Tasks;
using GxPress.Request.Reply;
using GxPress.Result.Reply;

namespace GxPress.Service.Interface.Reply
{
    public interface IReplyService : IService
    {
        /// <summary>
        /// 获取我收到和我回复的评论
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<IEnumerable<ReplyResult>> GetReplyResults(ReplyRequest request);
        /// <summary>
        /// 获取回复我的笔记或者话题
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<IEnumerable<ReplyResult>> GetReplyNoteOrTopicResults(ReplyRequest request);
        string GetTitleText(string content);
        /// <summary>
        /// 获取我收到的赞
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<IEnumerable<ReplyResult>> GetNotePraiseAsync(ReplyRequest request);

        /// <summary>
        /// 获取我收到的评论
        /// </summary>
        /// <returns></returns>
        Task<IEnumerable<ReplyResult>> GetReceiptCommentResult(ReplyRequest request);
        /// <summary>
        /// 获取我回复的评论
        /// </summary>
        /// <returns></returns>
        Task<IEnumerable<ReplyResult>> GetMyReceiptCommentResult(ReplyRequest reqeust);
        Task<IEnumerable<ReplyResult>> GetCommentPraiseAsync(ReplyRequest request);
        /// <summary>
        /// 获取通知点赞
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<IEnumerable<ReplyResult>> GetNoticePraiseAsync(ReplyRequest request);
        /// <summary>
        /// 获取话题点赞
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<IEnumerable<ReplyResult>> GetTopicPraiseAsync(ReplyRequest request);
        /// <summary>
        /// 未读点赞数量
        /// </summary>
        /// <param name="userId"></param>
        /// <returns></returns>
        Task<int> GetUReadPraiseCountAsync(int userId);
        /// <summary>
        /// 修改未读点赞数量
        /// </summary>
        /// <param name="userId"></param>
        /// <returns></returns>
        Task<bool> UpdateUReadPraiseAsync(int userId);
    }
}